@@ -1,6 +1,6 @@
 /*** date-core-strpf.c -- parser and formatter funs for date-core
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -66,34 +66,40 @@ DEFUN struct dt_d_s
 __strpd_std(const char *str, char **ep)
 {
 /* code dupe, see __strpdt_std() */
-	struct dt_d_s res = dt_d_initialiser();
-	struct strpd_s d;
+	struct dt_d_s res = {DT_DUNK};
+	struct strpd_s d = {0};
 	const char *sp;
 
 	if ((sp = str) == NULL) {
 		goto out;
 	}
 
-	d = strpd_initialiser();
+	d.c = -1;
 	/* read the year */
-	d.y = strtoi(sp, &sp);
+	d.y = strtoi32(sp, &sp);
 	if (d.y < DT_MIN_YEAR || d.y > DT_MAX_YEAR || *sp++ != '-') {
 		goto fucked;
 	}
 	/* check for ywd dates */
 	if (UNLIKELY(*sp == 'W')) {
 		/* brilliant */
-		sp++, d.c = strtoi(sp, &sp);
-		if (d.c < 0 || d.c > 53 || *sp++ != '-') {
+		sp++, d.c = strtoi32(sp, &sp);
+		if (d.c < 0 || d.c > 53) {
 			goto fucked;
 		}
 		d.flags.c_wcnt_p = 1;
 		d.flags.wk_cnt = YWD_ISOWK_CNT;
-		goto dow;
+		if (*sp == '-') {
+			goto dow;
+		} else if ((unsigned char)*sp <= ' ') {
+			/* unspecified week-date */
+			goto guess;
+		}
+		goto fucked;
 	}
 	/* read the month, then day count */
 	with (const char *tmp) {
-		d.m = strtoi(sp, &tmp);
+		d.m = strtoi32(sp, &tmp);
 		if (d.m < 0 || d.m > 366) {
 			goto fucked;
 		} else if (UNLIKELY(*tmp != '-')) {
@@ -107,8 +113,20 @@ __strpd_std(const char *str, char **ep)
 			} else {
 				goto fucked;
 			}
-			sp = tmp;
-		} else if (d.d = strtoi(++tmp, &sp), d.d < 0 || d.d > 31) {
+			switch (*(sp = tmp)) {
+			case 'B':
+				/* it's a bizda/YMDU before ultimo date */
+				d.flags.ab = BIZDA_BEFORE;
+			case 'b':
+				/* it's a bizda/YMDU after ultimo date */
+				d.flags.bizda = 1;
+				sp++;
+				/* we currently don't support
+				 * business day of year calendars */
+				goto fucked;
+			}
+			goto guess;
+		} else if (d.d = strtoi32(++tmp, &sp), d.d < 0 || d.d > 31) {
 			/* didn't work, fuck off */
 			goto fucked;
 		}
@@ -117,20 +135,18 @@ __strpd_std(const char *str, char **ep)
 	switch (*sp) {
 	case '-':
 		/* it is a YMCW date */
-		if ((d.c = d.d) > 5) {
+		if (d.d > 5) {
 			/* nope, it was bollocks */
-			break;
+			goto fucked;
 		}
-		d.d = 0;
-		sp++;
+		d.c = d.d, d.d = 0;
 	dow:
-		if (d.w = strtoi(sp, &sp),
+		sp++;
+		if (d.w = strtoi32(sp, &sp),
 		    d.w < 0 || (unsigned int)d.w > GREG_DAYS_P_WEEK) {
 			/* didn't work, fuck off */
 			goto fucked;
 		}
-		/* fix up d.w right away */
-		d.w = d.w ?: DT_SUNDAY;
 		break;
 	case 'B':
 		/* it's a bizda/YMDU before ultimo date */
@@ -146,6 +162,7 @@ __strpd_std(const char *str, char **ep)
 		/* we don't care */
 		break;
 	}
+guess:
 	/* guess what we're doing */
 	res = __guess_dtyp(d);
 out:
@@ -157,7 +174,7 @@ fucked:
 	if (ep != NULL) {
 		*ep = (char*)str;
 	}
-	return dt_d_initialiser();
+	return (struct dt_d_s){DT_DUNK};
 }
 
 DEFUN int
@@ -251,7 +268,6 @@ __strpd_card(struct strpd_s *d, const char *sp, struct dt_spec_s s, char **ep)
 		/* ymcw mode? */
 		d->w = strtoi_lim(sp, &sp, 0, GREG_DAYS_P_WEEK);
 		/* fix up d->w right away */
-		d->w = d->w ?: DT_SUNDAY;
 		res = 0 - (d->w < 0);
 		break;
 	case DT_SPFL_N_WCNT_MON:
@@ -539,10 +555,6 @@ __strfd_card(
 		with (unsigned int w = (unsigned)d->w ?: dt_get_wday(that)) {
 			const unsigned int ymcwp = s.wk_cnt != YWD_MONWK_CNT;
 
-			if (w == DT_SUNDAY && ymcwp) {
-				/* turn Sun 07 to Sun 00 */
-				w = 0;
-			}
 			res = ui99topstr(buf, bsz, w, 1 + ymcwp, padchar(s));
 		}
 		break;
