@@ -1,6 +1,6 @@
 /*** date-core.c -- our universe of dates
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -121,6 +121,12 @@ __uidiv(signed int x, signed int m)
 	return x >= 0 ? res : x % m ? res - 1 : res;
 }
 
+static inline __attribute__((const, pure)) int
+__sgn(signed int x)
+{
+	return (x > 0) - (x < 0);
+}
+
 
 /* helpers from the calendar files, don't define any aspect, so only
  * the helpers should get included */
@@ -790,7 +796,7 @@ __trans_ddurfmt(const char **fmt)
 DEFUN struct dt_d_s
 __guess_dtyp(struct strpd_s d)
 {
-	struct dt_d_s res = dt_d_initialiser();
+	struct dt_d_s res = {DT_DUNK};
 
 	if (LIKELY(d.y > 0 && d.c <= 0 && !d.flags.c_wcnt_p && !d.flags.bizda)) {
 		/* nearly all goes to ymd */
@@ -875,8 +881,8 @@ __guess_dtyp(struct strpd_s d)
 DEFUN struct dt_d_s
 dt_strpd(const char *str, const char *fmt, char **ep)
 {
-	struct dt_d_s res = dt_d_initialiser();
-	struct strpd_s d = strpd_initialiser();
+	struct dt_d_s res = {DT_DUNK};
+	struct strpd_s d = {0};
 	const char *sp = str;
 	const char *fp;
 
@@ -949,7 +955,7 @@ out:
 DEFUN size_t
 dt_strfd(char *restrict buf, size_t bsz, const char *fmt, struct dt_d_s that)
 {
-	struct strpd_s d = strpd_initialiser();
+	struct strpd_s d = {0};
 	const char *fp;
 	char *bp;
 	dt_dtyp_t tgttyp;
@@ -1148,7 +1154,7 @@ out:
 DEFUN size_t
 dt_strfddur(char *restrict buf, size_t bsz, const char *fmt, struct dt_ddur_s that)
 {
-	struct strpd_s d = strpd_initialiser();
+	struct strpd_s d = {0};
 	const char *fp;
 	char *bp;
 
@@ -1363,7 +1369,7 @@ dt_date(dt_dtyp_t outtyp)
 DEFUN struct dt_d_s
 dt_dconv(dt_dtyp_t tgttyp, struct dt_d_s d)
 {
-	struct dt_d_s res = dt_d_initialiser();
+	struct dt_d_s res = {DT_DUNK};
 
 	/* fix up before conversion */
 	d = dt_dfixup(d);
@@ -1422,7 +1428,7 @@ dt_dadd_d(struct dt_d_s d, int n)
 {
 /* add N (gregorian) days to D */
 	if (UNLIKELY(!n)) {
-		/* cacn't use short-cut return here, it'd upset the IPO/LTO */
+		/* can't use short-cut return here, it'd upset the IPO/LTO */
 		goto out;
 	}
 	switch (d.typ) {
@@ -1728,7 +1734,7 @@ DEFUN struct dt_d_s
 dt_dadd(struct dt_d_s d, struct dt_ddur_s dur)
 {
 	if (UNLIKELY(!dur.dv)) {
-		/* cacn't use short-cut return here, it'd upset the IPO/LTO */
+		/* can't use short-cut return here, it'd upset the IPO/LTO */
 		goto out;
 	}
 	switch (dur.durtyp) {
@@ -1741,6 +1747,9 @@ dt_dadd(struct dt_d_s d, struct dt_ddur_s dur)
 	case DT_DURWK:
 		d = dt_dadd_w(d, dur.dv);
 		break;
+	case DT_DURQU:
+		/* just use the month adder */
+		dur.dv *= 3U;
 	case DT_DURMO:
 		d = dt_dadd_m(d, dur.dv);
 		break;
@@ -1794,8 +1803,11 @@ dt_dur_neg_p(struct dt_ddur_s dur)
 }
 
 DEFUN struct dt_ddur_s
-dt_ddiff(dt_durtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2)
+dt_ddiff(dt_durtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2, int carry)
 {
+/* carry of 0 doesn't change anything
+ * carry <0 adds -1 to d2 iff d1 <= d2
+ * carry >0 adds 1 to d2 iff d1 >= d2 */
 	struct dt_ddur_s res = dt_make_ddur(DT_DURUNK, 0);
 
 	switch (tgttyp) {
@@ -1803,6 +1815,7 @@ dt_ddiff(dt_durtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2)
 	case DT_DURBD: {
 		dt_daisy_t tmp1 = dt_conv_to_daisy(d1);
 		dt_daisy_t tmp2 = dt_conv_to_daisy(d2);
+
 		res = __daisy_diff(tmp1, tmp2);
 
 		/* fix up result in case it's bizsi, i.e. kick weekends */
@@ -1815,25 +1828,61 @@ dt_ddiff(dt_durtyp_t tgttyp, struct dt_d_s d1, struct dt_d_s d2)
 	case DT_DURYMD: {
 		dt_ymd_t tmp1 = dt_conv_to_ymd(d1);
 		dt_ymd_t tmp2 = dt_conv_to_ymd(d2);
+		int fix = __sgn(carry) == -__sgn(tmp2.u - tmp1.u);
+
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __ymd_add_d(tmp2, __sgn(carry));
+		}
 		res = __ymd_diff(tmp1, tmp2);
+		res.fix = fix;
 		break;
 	}
 	case DT_DURYMCW: {
 		dt_ymcw_t tmp1 = dt_conv_to_ymcw(d1);
 		dt_ymcw_t tmp2 = dt_conv_to_ymcw(d2);
+		int fix = carry && __sgn(carry) == -__ymcw_cmp(tmp1, tmp2);
+
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __ymcw_add_d(tmp2, __sgn(carry));
+		}
 		res = __ymcw_diff(tmp1, tmp2);
+		res.fix = fix;
 		break;
 	}
 	case DT_DURYD: {
 		dt_yd_t tmp1 = dt_conv_to_yd(d1);
 		dt_yd_t tmp2 = dt_conv_to_yd(d2);
+		int fix = __sgn(carry) == -__sgn(tmp2.u - tmp1.u);
+
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __yd_add_d(tmp2, __sgn(carry));
+		}
 		res = __yd_diff(tmp1, tmp2);
+		res.fix = fix;
 		break;
 	}
 	case DT_DURYWD: {
 		dt_ywd_t tmp1 = dt_conv_to_ywd(d1);
 		dt_ywd_t tmp2 = dt_conv_to_ywd(d2);
+		int fix = __sgn(carry) == -__sgn(tmp2.u - tmp1.u);
+
+		if (UNLIKELY(fix)) {
+			/* add -1 iff tmp2 > tmp1 && carry < 0
+			 * add 1 iff tmp2 < tmp1 && carry > 0, i.e.
+			 * add sgn(carry) iff sgn(tmp2-tmp1) = -sgn(carry) */
+			tmp2 = __ywd_add_d(tmp2, __sgn(carry));
+		}
 		res = __ywd_diff(tmp1, tmp2);
+		res.fix = fix;
 		break;
 	}
 	case DT_DURBIZDA:
@@ -1894,7 +1943,7 @@ dt_d_in_range_p(struct dt_d_s d, struct dt_d_s d1, struct dt_d_s d2)
 	return 2 - ((m >> (i * 8U + j * 2U)) & 0b11U);
 }
 
-DEFUN __attribute__((pure)) struct dt_d_s
+DEFUN __attribute__((const)) struct dt_d_s
 dt_dfixup(struct dt_d_s d)
 {
 	switch (d.typ) {
