@@ -1,6 +1,6 @@
 /*** dt-core-strpf.c -- parser and formatter funs for dt-core
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -150,8 +150,8 @@ DEFUN struct dt_dt_s
 __strpdt_std(const char *str, char **ep)
 {
 /* code dupe, see __strpd_std() */
-	struct dt_dt_s res = dt_dt_initialiser();
-	struct strpdt_s d = strpdt_initialiser();
+	struct dt_dt_s res = {DT_UNK};
+	struct strpdt_s d = {0};
 	const char *sp;
 
 	if ((sp = str) == NULL) {
@@ -161,8 +161,8 @@ __strpdt_std(const char *str, char **ep)
 	if (*sp == '@') {
 		/* yay, epoch */
 		const char *tmp;
-		d.i = strtoi(++sp, &tmp);
-		if (UNLIKELY(d.i == -1 && sp == tmp)) {
+		d.i = strtoi64(++sp, &tmp);
+		if (UNLIKELY(d.i == INT64_MIN && sp == tmp)) {
 			sp--;
 		} else {
 			/* let's make a DT_SEXY */
@@ -277,16 +277,21 @@ __strpdt_card(struct strpdt_s *d, const char *sp, struct dt_spec_s s, char **ep)
 		res = __strpt_card(&d->st, sp, s, ep);
 		goto out_direct;
 
-	case DT_SPFL_N_EPOCH: {
+	case DT_SPFL_N_EPOCH:
+	case DT_SPFL_N_EPOCHNS: {
 		/* read over @ */
 		const char *tp = sp;
 		tp += *tp == '@';
-		d->i = strtoi(tp, &tp);
-		if (UNLIKELY(d->i < 0 || tp == sp)) {
+		d->i = strtoi64(tp, &tp);
+		if (UNLIKELY(d->i == INT64_MIN || tp == sp)) {
 			res = -1;
 		} else {
 			sp = tp;
 		}
+		if (s.spfl == DT_SPFL_N_EPOCHNS) {
+			d->st.ns = d->i % 1000000000LL;
+			d->i /= 1000000000LL;
+		}
 		break;
 	}
 
@@ -358,7 +363,8 @@ __strfdt_card(
 		res = __strft_card(buf, bsz, s, &d->st, that.t);
 		break;
 
-	case DT_SPFL_N_EPOCH: {
+	case DT_SPFL_N_EPOCH:
+	case DT_SPFL_N_EPOCHNS: {
 		/* convert to sexy */
 		int64_t sexy = dt_conv_to_sexy(that).sexy;
 		res = snprintf(buf, bsz, "%" PRIi64, sexy);
