@@ -1,6 +1,6 @@
 /*** dt-core.c -- our universe of datetimes
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -85,7 +85,7 @@
 struct strpdt_s {
 	struct strpd_s sd;
 	struct strpt_s st;
-	long int i;
+	int64_t i;
 
 	/* use 31 bits for the difference */
 	int32_t zdiff:31;
@@ -185,7 +185,7 @@ dt_conv_to_sexy(struct dt_dt_s dt)
 	} else if (dt_sandwich_p(dt) || dt_sandwich_only_d_p(dt)) {
 		dt.sxepoch = __to_unix_epoch(dt);
 	} else {
-		dt = dt_dt_initialiser();
+		dt = (struct dt_dt_s){DT_UNK};
 	}
 	/* make sure we hand out sexies */
 	dt.typ = DT_SEXY;
@@ -195,15 +195,29 @@ dt_conv_to_sexy(struct dt_dt_s dt)
 static inline struct dt_dt_s
 __sexy_to_daisy(dt_ssexy_t sx)
 {
-	struct dt_dt_s res = dt_dt_initialiser();
+	struct dt_dt_s res = {DT_UNK};
+	int s, m, h;
 
-	res.t.hms.s = sx % SECS_PER_MIN;
+	s = sx % SECS_PER_MIN;
 	sx /= SECS_PER_MIN;
-	res.t.hms.m = sx % MINS_PER_HOUR;
+	m = sx % MINS_PER_HOUR;
 	sx /= MINS_PER_HOUR;
-	res.t.hms.h = sx % HOURS_PER_DAY;
+	h = sx % HOURS_PER_DAY;
 	sx /= HOURS_PER_DAY;
 
+	m -= s < 0;
+	h -= m < 0;
+	sx -= h < 0;
+
+	s += s >= 0 ? 0 : SECS_PER_MIN;
+	m += m >= 0 ? 0 : MINS_PER_HOUR;
+	h += h >= 0 ? 0 : HOURS_PER_DAY;
+
+	/* assign now */
+	res.t.hms.s = s;
+	res.t.hms.m = m;
+	res.t.hms.h = h;
+
 	/* rest is a day-count, move to daisy */
 	res.d.daisy = sx + DAISY_UNIX_BASE;
 
@@ -215,7 +229,7 @@ __sexy_to_daisy(dt_ssexy_t sx)
 static inline struct dt_dt_s
 __ymdhms_to_ymd(dt_ymdhms_t x)
 {
-	struct dt_dt_s res = dt_dt_initialiser();
+	struct dt_dt_s res = {DT_UNK};
 
 	res.t.hms.s = x.S;
 	res.t.hms.m = x.M;
@@ -442,7 +456,7 @@ now_tm(void)
 	} else if ((tv = now_tv()).tv_sec == 0U) {
 		/* big cinema :( */
 #if defined HAVE_SLOPPY_STRUCTS_INIT
-		return (struct tm){};
+		return (struct tm){0};
 #else  /* !HAVE_SLOPPY_STRUCTS_INIT */
 		memset(&tm, 0, sizeof(tm));
 #endif	/* HAVE_SLOPPY_STRUCTS_INIT */
@@ -458,11 +472,7 @@ massage_strpdt(struct strpdt_s d)
 /* the reason we do this separately is that we don't want to bother
  * the pieces of code that use the guesser for different reasons */
 	if (UNLIKELY(d.sd.y == 0U)) {
-#if defined HAVE_SLOPPY_STRUCTS_INIT
-		static const struct strpd_s d0 = {};
-#else  /* !HAVE_SLOPPY_STRUCTS_INIT */
-		static const struct strpd_s d0;
-#endif	/* HAVE_SLOPPY_STRUCTS_INIT */
+		static const struct strpd_s d0 = {0};
 		struct dt_dt_s now = dt_get_base();
 
 		if (UNLIKELY(memcmp(&d.sd, &d0, sizeof(d0)) == 0U)) {
@@ -568,7 +578,7 @@ zdiff_sec(struct dt_dt_s d)
 	return zdiff;
 }
 
-static inline __attribute__((const, pure)) bool
+static inline __attribute__((const)) bool
 dt_dur_only_d_p(struct dt_dtdur_s d)
 {
 	return d.durtyp && d.d.durtyp < DT_NDURTYP && !d.t.sdur;
@@ -601,8 +611,8 @@ need_milfup_p(const char *fmt)
 DEFUN struct dt_dt_s
 dt_strpdt(const char *str, const char *fmt, char **ep)
 {
-	struct dt_dt_s res = dt_dt_initialiser();
-	struct strpdt_s d;
+	struct dt_dt_s res = {DT_UNK};
+	struct strpdt_s d = {0};
 	const char *sp = str;
 	const char *fp;
 
@@ -633,7 +643,7 @@ dt_strpdt(const char *str, const char *fmt, char **ep)
 		goto sober;
 
 	case DT_LDN:
-		res.d.ldn = (dt_ldn_t)strtoi(str, &sp);
+		res.d.ldn = (dt_ldn_t)strtoi32(str, &sp);
 		if (*sp == '.') {
 			/* oooh, a double it seems */
 			double tmp = strtod(sp, &on);
@@ -659,7 +669,7 @@ dt_strpdt(const char *str, const char *fmt, char **ep)
 		goto sober;
 
 	case DT_MDN:
-		res.d.mdn = (dt_ldn_t)strtoi(str, &sp);
+		res.d.mdn = (dt_ldn_t)strtoi32(str, &sp);
 		if (*sp == '.') {
 			/* oooh, a double it seems */
 			double tmp = strtod(sp, &on);
@@ -686,14 +696,13 @@ dt_strpdt(const char *str, const char *fmt, char **ep)
 	}
 
 	fp = fmt;
-	d = strpdt_initialiser();
 	while (*fp && *sp) {
 		const char *fp_sav = fp;
 		struct dt_spec_s spec = __tok_spec(fp_sav, &fp);
 
 		if (spec.spfl == DT_SPFL_UNK) {
 			/* must be literal */
-			if (*fp_sav != *sp++) {
+			if (UNLIKELY(*fp_sav != *sp++)) {
 				goto fucked;
 			}
 		} else if (LIKELY(!spec.rom)) {
@@ -765,13 +774,13 @@ fucked:
 	if (ep != NULL) {
 		*ep = (char*)str;
 	}
-	return dt_dt_initialiser();
+	return (struct dt_dt_s){DT_UNK};
 }
 
 DEFUN size_t
 dt_strfdt(char *restrict buf, size_t bsz, const char *fmt, struct dt_dt_s that)
 {
-	struct strpdt_s d = strpdt_initialiser();
+	struct strpdt_s d = {0};
 	const char *fp;
 	char *bp;
 	dt_dtyp_t tgttyp;
@@ -1093,7 +1102,7 @@ DEFUN size_t
 dt_strfdtdur(
 	char *restrict buf, size_t bsz, const char *fmt, struct dt_dtdur_s that)
 {
-	struct strpdt_s d;
+	struct strpdt_s d = {0};
 	const char *fp;
 	char *bp;
 
@@ -1102,7 +1111,6 @@ dt_strfdtdur(
 		goto out;
 	}
 
-	d = strpdt_initialiser();
 	switch (that.d.durtyp) {
 	case DT_YMD:
 		d.sd.y = that.d.ymd.y;
@@ -1295,7 +1303,7 @@ dt_dtdur_neg_p(struct dt_dtdur_s dur)
 DEFUN struct dt_dt_s
 dt_datetime(dt_dttyp_t outtyp)
 {
-	struct dt_dt_s res = dt_dt_initialiser();
+	struct dt_dt_s res = {DT_UNK};
 	const dt_dtyp_t outdtyp = (dt_dtyp_t)outtyp;
 	struct tm tm = now_tm();
 	struct timeval tv = now_tv();
@@ -1407,7 +1415,7 @@ dt_dtconv(dt_dttyp_t tgttyp, struct dt_dt_s d)
 			unsigned int ss = __secs_since_midnight(d.t);
 
 			switch (tgttyp) {
-				int32_t sx;
+				int64_t sx;
 #if defined WITH_LEAP_SECONDS
 			case DT_SEXYTAI: {
 				zidx_t zi;
@@ -1437,7 +1445,7 @@ dt_dtconv(dt_dttyp_t tgttyp, struct dt_dt_s d)
 
 		case DT_DUNK:
 		default:
-			d = dt_dt_initialiser();
+			d = (struct dt_dt_s){DT_UNK};
 			break;
 		}
 	} else if (dt_sandwich_only_t_p(d)) {
@@ -1467,7 +1475,7 @@ dt_dtconv(dt_dttyp_t tgttyp, struct dt_dt_s d)
 			}
 			break;
 		default:
-			d = dt_dt_initialiser();
+			d = (struct dt_dt_s){DT_UNK};
 			break;
 		}
 	} else {
@@ -1544,7 +1552,9 @@ dt_dtadd(struct dt_dt_s d, struct dt_dtdur_s dur)
 			carry--;
 		}
 
-		d.t.hms.ns = dv;
+		if (d.sandwich) {
+			d.t.hms.ns = dv;
+		}
 
 		/* the rest is normal second-wise tadd */
 		dv = carry;
@@ -1596,35 +1606,48 @@ DEFUN struct dt_dtdur_s
 dt_dtdiff(dt_dtdurtyp_t tgttyp, struct dt_dt_s d1, struct dt_dt_s d2)
 {
 	struct dt_dtdur_s res = {(dt_dtdurtyp_t)DT_DURUNK};
-	int dt = 0;
+	int64_t dt = 0;
 
 	if (!dt_sandwich_only_d_p(d1) && !dt_sandwich_only_d_p(d2)) {
 		/* do the time portion difference right away */
-		dt = dt_tdiff_s(d1.t, d2.t);
+		switch (tgttyp) {
+		default:
+			dt = dt_tdiff_s(d1.t, d2.t);
+			break;
+		case DT_DURNANO:
+			dt = dt_tdiff_ns(d1.t, d2.t);
+			break;
+		}
 	}
 	/* now assess what else is to be done */
 	if (dt_sandwich_only_t_p(d1) && dt_sandwich_only_t_p(d2)) {
 		/* make t-only */
-		res.durtyp = DT_DURS;
-		res.dv = dt;
+		res.durtyp = (dt_dtdurtyp_t)(DT_DURS + (tgttyp == DT_DURNANO));
+		res.neg = (uint16_t)(dt < 0);
+		res.dv = dt >= 0 ? dt : -dt;
 	} else if (tgttyp && (dt_durtyp_t)tgttyp < DT_NDURTYP) {
-		/* check for negative carry */
-		if (UNLIKELY(dt < 0)) {
-			d2.d = dt_dadd(d2.d, dt_make_ddur(DT_DURD, -1));
-			dt += SECS_PER_DAY;
-		}
 		res.durtyp = tgttyp;
-		res.d = dt_ddiff((dt_durtyp_t)tgttyp, d1.d, d2.d);
+		res.d = dt_ddiff((dt_durtyp_t)tgttyp, d1.d, d2.d, dt);
+		dt = !res.neg ? dt : -dt;
+		dt = !res.d.fix ? dt
+			: dt > 0 ? SECS_PER_DAY - dt
+			: dt < 0 ? dt + SECS_PER_DAY
+			: 0;
 		res.t.sdur = dt;
+		res.t.nsdur = 0;
 	} else if ((dt_durtyp_t)tgttyp >= DT_NDURTYP) {
 		int64_t sxdur;
 
 		if (d1.typ < DT_PACK && d2.typ < DT_PACK) {
 			/* go for tdiff and ddiff independently */
-			res.d = dt_ddiff(DT_DURD, d1.d, d2.d);
+			res.d = dt_ddiff(DT_DURD, d1.d, d2.d, 0);
+
+			if (UNLIKELY(tgttyp == DT_DURNANO)) {
+				/* unfortunately we have to scale back */
+				dt /= NANOS_PER_SEC;
+			}
 			/* since target type is SEXY do the conversion here */
-			sxdur = (int64_t)dt +
-				(int64_t)res.d.dv * SECS_PER_DAY;
+			sxdur = dt + (int64_t)res.d.dv * SECS_PER_DAY;
 		} else {
 			/* oh we're in the sexy domain already,
 			 * note, we can't diff ymdhms packs */
@@ -1632,18 +1655,17 @@ dt_dtdiff(dt_dtdurtyp_t tgttyp, struct dt_dt_s d1, struct dt_dt_s d2)
 			d2 = dt_dtconv(DT_SEXY, d2);
 
 			/* now it's fuck-easy */
-			sxdur = d2.sexy - d1.sexy;
+			sxdur = (int64_t)(d2.sexy - d1.sexy);
 		}
 
 		/* set up the output here */
-		res.durtyp = tgttyp;
+		res.durtyp = DT_DURS;
 		res.neg = 0U;
 		if (LIKELY(tgttyp < DT_NDTDURTYP)) {
 			res.tai = 0U;
 		} else {
 			/* just a hack to transfer the notion of TAIness */
 			res.tai = 1U;
-			res.durtyp = DT_DURS;
 		}
 		res.dv = sxdur;
 
@@ -1800,7 +1822,7 @@ dt_get_tbase(void)
 # endif	/* INCLUDED_time_core_h_ */
 #endif	/* LIBDUT */
 
-DEFUN __attribute__((pure)) struct dt_dt_s
+DEFUN __attribute__((const)) struct dt_dt_s
 dt_fixup(struct dt_dt_s d)
 {
 	if (LIKELY(dt_sandwich_only_d_p(d) || dt_sandwich_p(d))) {
@@ -1809,7 +1831,7 @@ dt_fixup(struct dt_dt_s d)
 	return d;
 }
 
-DEFUN __attribute__((pure)) struct dt_dt_s
+DEFUN __attribute__((const)) struct dt_dt_s
 dt_milfup(struct dt_dt_s dt)
 {
 	if (dt_sandwich_p(dt) && UNLIKELY(dt.t.hms.h == 24)) {
