@@ -1,6 +1,6 @@
 /*** strops.c -- useful string operations
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -109,11 +109,11 @@ padstrtoi_lim(const char *str, const char **ep, int32_t llim, int32_t ulim)
 }
 
 DEFUN int32_t
-strtoi(const char *str, const char **ep)
+strtoi32(const char *str, const char **ep)
 {
 	const char *sp = str;
 	bool negp = false;
-	int32_t res = 0;
+	int32_t res = INT32_MIN;
 
 	if (*str == '-') {
 		negp = true;
@@ -122,12 +122,32 @@ strtoi(const char *str, const char **ep)
 	while (res < INT32_MAX / 10 && (unsigned char)(*sp ^ '0') < 10U) {
 		res *= 10, res += (unsigned char)(*sp++ ^ '0');
 	}
-	if (UNLIKELY(sp == str)) {
-		res = -1;
-	} else if (negp) {
+	if (negp) {
 		res = -res;
 	}
-	*ep = (char*)sp;
+	*ep = res > INT32_MIN ? (char*)sp : (char*)str;
+	return res;
+}
+
+
+DEFUN int64_t
+strtoi64(const char *str, const char **ep)
+{
+	const char *sp = str;
+	bool negp = false;
+	int64_t res = INT64_MIN;
+
+	if (*str == '-') {
+		negp = true;
+		sp++;
+	}
+	while (res < INT64_MAX / 10 && (unsigned char)(*sp ^ '0') < 10U) {
+		res *= 10, res += (unsigned char)(*sp++ ^ '0');
+	}
+	if (negp) {
+		res = -res;
+	}
+	*ep = res > INT64_MIN ? (char*)sp : (char*)str;
 	return res;
 }
 
