@@ -1,6 +1,6 @@
 /*** token.c -- tokeniser specs and stuff
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -56,7 +56,7 @@
 struct dt_spec_s
 __tok_spec(const char *fp, const char **ep)
 {
-	struct dt_spec_s res = spec_initialiser();
+	struct dt_spec_s res = {0};
 
 	if (*fp != '%') {
 		goto out;
@@ -146,6 +146,10 @@ next:
 		break;
 	case 's':
 		res.spfl = DT_SPFL_N_EPOCH;
+		if (fp[1U] == '%' && fp[2U] == 'N') {
+			res.spfl = DT_SPFL_N_EPOCHNS;
+			fp += 2U;
+		}
 		break;
 	case 'Z':
 		res.spfl = DT_SPFL_N_ZDIFF;
