@@ -1,6 +1,6 @@
 /*** token.h -- tokeniser specs and stuff
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -85,6 +85,8 @@ typedef enum {
 	/* date/time specs */
 	/* %s epoch spec, non-standard */
 	DT_SPFL_N_EPOCH,
+	/* %s%N epoch spec, non-standard */
+	DT_SPFL_N_EPOCHNS,
 	/* %Z zone difference */
 	DT_SPFL_N_ZDIFF,
 	DT_SPFL_N_DTLAST = DT_SPFL_N_ZDIFF,
@@ -155,18 +157,7 @@ struct dt_spec_s {
 extern struct dt_spec_s __tok_spec(const char *fp, const char **ep);
 
 
-static inline __attribute__((pure, const)) struct dt_spec_s
-spec_initialiser(void)
-{
-#if defined HAVE_SLOPPY_STRUCTS_INIT
-	static const struct dt_spec_s res = {};
-#else
-	static const struct dt_spec_s res;
-#endif	/* HAVE_SLOPPY_STRUCTS_INIT */
-	return res;
-}
-
-static inline __attribute__((pure, const)) char
+static inline __attribute__((const)) char
 padchar(struct dt_spec_s s)
 {
 	return (char)(0x20U ^ ((s.pad < DT_SPPAD_SPC) << 4U));
