@@ -1,6 +1,6 @@
 /*** yd.c -- guts for yd dates that consist of a year and a year-day
  *
- * Copyright (C) 2010-2018 Sebastian Freundt
+ * Copyright (C) 2010-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -141,13 +141,13 @@ static const __jan01_wday_block_t __jan01_wday[] = {
 # undef A
 # undef S
 
-static inline __attribute__((const, pure)) __jan01_wday_block_t
+static inline __attribute__((pure)) __jan01_wday_block_t
 __get_jan01_block(unsigned int year)
 {
 	return __jan01_wday[(year - __JAN01_WDAY_BEG) / __JAN01_Y_PER_B];
 }
 
-static inline __attribute__((const, pure)) dt_dow_t
+static inline __attribute__((const)) dt_dow_t
 __get_jan01_wday(unsigned int year)
 {
 /* get the weekday of jan01 in YEAR */
@@ -229,7 +229,7 @@ static const dt_dow_t __jan01_28y_wday[] = {
 # undef A
 # undef S
 
-static __attribute__((const, pure)) unsigned int
+static __attribute__((const)) unsigned int
 __get_28y_year_equiv(unsigned year)
 {
 /* the 28y cycle works for 1901 to 2100, for other years find an equivalent */
@@ -245,7 +245,7 @@ __get_28y_year_equiv(unsigned year)
 	return year + 2000;
 }
 
-static inline __attribute__((const, pure)) dt_dow_t
+static inline __attribute__((pure)) dt_dow_t
 __get_jan01_wday(unsigned int year)
 {
 /* get the weekday of jan01 in YEAR
@@ -269,7 +269,7 @@ __get_jan01_wday(unsigned int year)
 
 #elif defined GET_JAN01_WDAY_28Y_SWITCH
 
-static inline __attribute__((const, pure)) dt_dow_t
+static inline __attribute__((const)) dt_dow_t
 __get_jan01_wday(unsigned int year)
 {
 /* get the weekday of jan01 in YEAR
@@ -354,7 +354,7 @@ __get_jan01_wday(unsigned int year)
 
 #elif defined GET_JAN01_WDAY_SAKAMOTO
 
-static inline __attribute__((const, pure)) dt_dow_t
+static inline __attribute__((const)) dt_dow_t
 __get_jan01_wday(unsigned int year)
 {
 	unsigned int res;
@@ -368,7 +368,7 @@ __get_jan01_wday(unsigned int year)
 #endif	/* GET_JAN01_WDAY_* */
 
 #if defined YMD_GET_YD_LOOKUP
-static inline __attribute__((const, pure)) unsigned int
+static inline __attribute__((pure)) unsigned int
 __md_get_yday(unsigned int year, unsigned int mon, unsigned int dom)
 {
 	static uint16_t __mon_yday[] = {
@@ -382,7 +382,7 @@ __md_get_yday(unsigned int year, unsigned int mon, unsigned int dom)
 }
 
 #elif defined YMD_GET_YD_DIVREM
-static inline __attribute__((const, pure)) unsigned int
+static inline __attribute__((const)) unsigned int
 __md_get_yday(unsigned int year, unsigned int mon, unsigned int dom)
 {
 #define SL(x)	((x) * 5)
@@ -410,7 +410,7 @@ __md_get_yday(unsigned int year, unsigned int mon, unsigned int dom)
 
 #if defined GET_MD_FREUNDT
 /* Freundt's 32-adic algo */
-static __attribute__((const, pure)) struct __md_s
+static __attribute__((pure)) struct __md_s
 __yday_get_md(unsigned int year, unsigned int doy)
 {
 /* Given a year and the day of the year, return gregorian month + dom
@@ -437,7 +437,6 @@ __yday_get_md(unsigned int year, unsigned int doy)
 #define GET_REM(x)	(rem[x])
 	static const uint8_t rem[] = {
 		19, 19, 18, 14, 13, 11, 10, 8, 7, 6, 4, 3, 1, 0
-
 	};
 	unsigned int m;
 	unsigned int d;
@@ -468,7 +467,7 @@ __yday_get_md(unsigned int year, unsigned int doy)
 
 #elif defined GET_MD_TRIMESTER
 /* Trimester algo, we can't figure out which one's faster */
-static __attribute__((const, pure)) struct __md_s
+static __attribute__((const)) struct __md_s
 __yday_get_md(unsigned int year, unsigned int yday)
 {
 /* The idea here is that the year can be divided into trimesters:
@@ -518,14 +517,14 @@ __yday_get_md(unsigned int year, unsigned int yday)
 }
 #endif	/* GET_MD_* */
 
-static inline __attribute__((const, pure)) dt_dow_t
+static inline __attribute__((const)) dt_dow_t
 __get_jan01_yday_dow(unsigned int yd, dt_dow_t w)
 {
 	unsigned int res = (yd + 6U - (unsigned int)w) % GREG_DAYS_P_WEEK;
 	return (dt_dow_t)(DT_SUNDAY - res);
 }
 
-static inline __attribute__((const, pure)) unsigned int
+static inline __attribute__((const)) unsigned int
 __get_ydays(unsigned int y)
 {
 	return LIKELY(!__leapp(y)) ? 365U : 366U;
@@ -549,7 +548,7 @@ __yd_fixup(dt_yd_t d)
 
 #if defined ASPECT_GETTERS && !defined YD_ASPECT_GETTERS_
 #define YD_ASPECT_GETTERS_
-static inline __attribute__((const, pure)) int
+static inline __attribute__((pure)) int
 __get_isowk_wd(unsigned int yd, dt_dow_t f01)
 {
 /* given the weekday the year starts with, F01, and the year-day YD
@@ -558,7 +557,7 @@ __get_isowk_wd(unsigned int yd, dt_dow_t f01)
 	return (GREG_DAYS_P_WEEK + yd - iso[f01]) / GREG_DAYS_P_WEEK;
 }
 
-DEFUN __attribute__((const, pure)) int
+DEFUN __attribute__((const)) int
 __yd_get_wcnt_abs(dt_yd_t d)
 {
 /* absolutely count the n-th occurrence of WD regardless what WD
@@ -568,7 +567,7 @@ __yd_get_wcnt_abs(dt_yd_t d)
 	return (GREG_DAYS_P_WEEK + yd - 1) / GREG_DAYS_P_WEEK;
 }
 
-DEFUN __attribute__((const, pure)) int
+DEFUN __attribute__((const)) int
 __yd_get_wcnt_iso(dt_yd_t d)
 {
 /* like __yd_get_wcnt() but for iso week conventions
@@ -618,7 +617,7 @@ __yd_get_wcnt_iso(dt_yd_t d)
 	return wk;
 }
 
-DEFUN __attribute__((const, pure)) int
+DEFUN __attribute__((const)) int
 __yd_get_wcnt(dt_yd_t d, dt_dow_t _1st_wd)
 {
 /* absolutely count the n-th occurrence of WD regardless what WD
@@ -756,7 +755,7 @@ static dt_yd_t
 __yd_add_d(dt_yd_t d, int n)
 {
 /* add N days to D */
-	signed int tgtd = d.d + n;
+	signed int tgtd = d.d + n + (n < 0 && !d.d);
 
 	/* fixup the day */
 	return __yd_fixup_d(d.y, tgtd);
@@ -766,8 +765,12 @@ static dt_yd_t
 __yd_add_b(dt_yd_t d, int n)
 {
 /* add N business days to D */
-	dt_dow_t wd = __yd_get_wday(d);
-	int tgtd = d.d + __get_d_equiv(wd, n);
+	dt_dow_t wd;
+	signed int tgtd;
+
+	d.d ^= n < 0 && !d.d;
+	wd = __yd_get_wday(d);
+	tgtd = d.d + __get_d_equiv(wd, n);
 
 	/* fixup the day, i.e. 2012-0367 -> 2013-001 */
 	return __yd_fixup_d(d.y, tgtd);
@@ -792,7 +795,7 @@ __yd_add_y(dt_yd_t d, int n)
 
 #if defined ASPECT_DIFF && !defined YD_ASPECT_DIFF_
 #define YD_ASPECT_DIFF_
-static __attribute__((const, pure)) struct dt_ddur_s
+static __attribute__((const)) struct dt_ddur_s
 __yd_diff(dt_yd_t d1, dt_yd_t d2)
 {
 /* compute d2 - d1 entirely in terms of ymd but express the result as yd */
