@@ -1,6 +1,6 @@
 /*** dadd.c -- perform simple date arithmetic, date plus duration
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -83,6 +83,7 @@ struct mass_add_clo_s {
 	zif_t z;
 	const char *ofmt;
 	int sed_mode_p;
+	int empty_mode_p;
 	int quietp;
 };
 
@@ -92,6 +93,7 @@ proc_line(const struct mass_add_clo_s *clo, char *line, size_t llen)
 	struct dt_dt_s d;
 	char *sp = NULL;
 	char *ep = NULL;
+	size_t nmatch = 0U;
 	int rc = 0;
 
 	do {
@@ -116,14 +118,19 @@ proc_line(const struct mass_add_clo_s *clo, char *line, size_t llen)
 				dt_io_write(d, clo->ofmt, clo->z, '\0');
 				llen -= (ep - line);
 				line = ep;
+				nmatch++;
 			} else {
 				dt_io_write(d, clo->ofmt, clo->z, '\n');
 				break;
 			}
 		} else if (clo->sed_mode_p) {
+			llen = !(clo->empty_mode_p && !nmatch) ? llen : 0U;
 			line[llen] = '\n';
 			__io_write(line, llen + 1, stdout);
 			break;
+		} else if (clo->empty_mode_p) {
+			__io_write("\n", 1U, stdout);
+			break;
 		} else {
 			/* obviously unmatched, warn about it in non -q mode */
 			if (!clo->quietp) {
@@ -163,7 +170,7 @@ mass_add_d(const struct mass_add_clo_s *clo)
  * output */
 	size_t lno = 0;
 	struct dt_dt_s d;
-	struct __strpdtdur_st_s st = __strpdtdur_st_initialiser();
+	struct __strpdtdur_st_s st = {0};
 	int rc = 0;
 
 	for (char *line; prchunk_haslinep(clo->pctx); lno++) {
@@ -219,7 +226,7 @@ main(int argc, char *argv[])
 {
 	yuck_t argi[1U];
 	struct dt_dt_s d;
-	struct __strpdtdur_st_s st = __strpdtdur_st_initialiser();
+	struct __strpdtdur_st_s st = {0};
 	const char *ofmt;
 	char **fmt;
 	size_t nfmt;
@@ -288,7 +295,7 @@ main(int argc, char *argv[])
 				serror("Error: \
 cannot parse duration string `%s'", st.istr);
 				rc = 1;
-				goto dur_out;
+				goto clear;
 			}
 		} while (__strpdtdur_more_p(&st));
 	}
@@ -303,7 +310,7 @@ cannot parse duration string `%s'", st.istr);
 			error("\
 Error: cannot interpret date/time string `%s'", inp);
 			rc = 1;
-			goto dur_out;
+			goto clear;
 		}
 	}
 
@@ -322,6 +329,50 @@ Error: cannot interpret date/time string `%s'", inp);
 			rc = 1;
 		}
 
+	} else if (st.ndurs && !argi->sed_mode_flag && argi->empty_mode_flag) {
+		size_t lno = 0U;
+		void *pctx;
+
+		/* no threads reading this stream */
+		__io_setlocking_bycaller(stdout);
+
+		/* using the prchunk reader now */
+		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
+			serror("could not open stdin");
+			goto clear;
+		}
+
+		while (prchunk_fill(pctx) >= 0) {
+			for (char *line; prchunk_haslinep(pctx); lno++) {
+				size_t llen = prchunk_getline(pctx, &line);
+				char *ep = NULL;
+
+				if (UNLIKELY(!llen)) {
+					goto empty;
+				}
+				/* try and parse the line */
+				d = dt_io_strpdt_ep(line, fmt, nfmt, &ep, fromz);
+				if (UNLIKELY(dt_unk_p(d))) {
+					goto empty;
+				} else if (ep && (unsigned)*ep >= ' ') {
+					goto empty;
+				}
+				/* do the adding */
+				d = dadd_add(d, st.durs, st.ndurs);
+				if (UNLIKELY(dt_unk_p(d))) {
+					goto empty;
+				}
+
+				if (hackz == NULL && fromz != NULL) {
+					/* fixup zone */
+					d = dtz_forgetz(d, fromz);
+				}
+				dt_io_write(d, ofmt, z, '\n');
+				continue;
+			empty:
+				__io_write("\n", 1U, stdout);
+			}
+		}
 	} else if (st.ndurs) {
 		/* read dates from stdin */
 		struct grep_atom_s __nstk[16], *needle = __nstk;
@@ -357,6 +408,7 @@ Error: cannot interpret date/time string `%s'", inp);
 		clo->z = z;
 		clo->ofmt = ofmt;
 		clo->sed_mode_p = argi->sed_mode_flag;
+		clo->empty_mode_p = argi->empty_mode_flag;
 		clo->quietp = argi->quiet_flag;
 		while (prchunk_fill(pctx) >= 0) {
 			rc |= mass_add_dur(clo);
@@ -379,7 +431,7 @@ Error: cannot interpret date/time string `%s'", inp);
 		/* using the prchunk reader now */
 		if ((pctx = init_prchunk(STDIN_FILENO)) == NULL) {
 			serror("could not open stdin");
-			goto dur_out;
+			goto clear;
 		}
 
 		/* build the clo and then loop */
@@ -390,6 +442,7 @@ Error: cannot interpret date/time string `%s'", inp);
 		clo->z = z;
 		clo->ofmt = ofmt;
 		clo->sed_mode_p = argi->sed_mode_flag;
+		clo->empty_mode_p = argi->empty_mode_flag;
 		clo->quietp = argi->quiet_flag;
 		while (prchunk_fill(pctx) >= 0) {
 			rc |= mass_add_d(clo);
@@ -397,7 +450,7 @@ Error: cannot interpret date/time string `%s'", inp);
 		/* get rid of resources */
 		free_prchunk(pctx);
 	}
-dur_out:
+clear:
 	/* free the strpdur status */
 	__strpdtdur_free(&st);
 
