@@ -1,6 +1,6 @@
 /*** ddiff.c -- perform simple date arithmetic, date minus date
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -57,6 +57,7 @@ typedef union {
 	unsigned int flags;
 	struct {
 		unsigned int has_year:1;
+		unsigned int has_qtr:1;
 		unsigned int has_mon:1;
 		unsigned int has_week:1;
 		unsigned int has_day:1;
@@ -141,6 +142,9 @@ determine_durfmt(const char *fmt)
 			case DT_SPFL_N_YEAR:
 				res.has_year = 1;
 				break;
+			case DT_SPFL_N_QTR:
+				res.has_qtr = 1;
+				break;
 			case DT_SPFL_N_MON:
 			case DT_SPFL_S_MON:
 				res.has_mon = 1;
@@ -200,11 +204,11 @@ determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 	} else if (dt_sandwich_only_t_p(d1) || dt_sandwich_only_t_p(d2)) {
 		/* isn't defined */
 		return (dt_dtdurtyp_t)DT_DURUNK;
-	} else if (f.has_week && f.has_mon) {
+	} else if (f.has_week && (f.has_mon || f.has_qtr)) {
 		return (dt_dtdurtyp_t)DT_DURYMD;
 	} else if (f.has_week && f.has_year) {
 		return (dt_dtdurtyp_t)DT_DURYWD;
-	} else if (f.has_mon) {
+	} else if (f.has_mon || f.has_qtr) {
 		return (dt_dtdurtyp_t)DT_DURYMD;
 	} else if (f.has_year && f.has_day) {
 		return (dt_dtdurtyp_t)DT_DURYD;
@@ -220,7 +224,7 @@ determine_durtype(struct dt_dt_s d1, struct dt_dt_s d2, durfmt_t f)
 		return (dt_dtdurtyp_t)0xffU;
 	}
 	/* otherwise */
-	return DT_DURS;
+	return (dt_dtdurtyp_t)(DT_DURS + (f.has_nano));
 }
 
 
@@ -282,7 +286,7 @@ static __attribute__((pure)) long int
 __strf_tot_secs(struct dt_dtdur_s dur)
 {
 /* return time portion of duration in UTC seconds */
-	long int s = dur.dv;
+	int64_t s = dur.dv;
 
 	if (UNLIKELY(dur.tai) && dur.durtyp == DT_DURS) {
 		return dur.soft - dur.corr;
@@ -392,7 +396,9 @@ __strf_tot_years(struct dt_dtdur_s dur)
 }
 
 static struct precalc_s {
+	int neg;
 	int Y;
+	int q;
 	int m;
 	int w;
 	int d;
@@ -418,64 +424,53 @@ static struct precalc_s {
 		/* just years */
 		res.Y = __strf_tot_years(dur);
 	}
-	if (f.has_year && f.has_mon) {
+	if (f.has_year && (f.has_mon || f.has_qtr)) {
 		/* years and months */
 		res.m = __strf_ym_mon(dur);
-	} else if (f.has_mon) {
+	} else if (f.has_mon || f.has_qtr) {
 		/* just months */
 		res.m = __strf_tot_mon(dur);
 	}
 
+	if (f.has_qtr) {
+		/* split m slot */
+		res.q = res.m / 3;
+		res.m = res.m % 3;
+	}
+
 	/* the other units are easily converted as their factors are fixed.
 	 * we operate on clean seconds and attribute leap seconds only
 	 * to the S slot, so 59 seconds plus a leap second != 1 minute */
-	with (long long int S = __strf_tot_secs(dur), d = __strf_tot_days(dur)) {
+	with (int64_t S = __strf_tot_secs(dur), d = __strf_tot_days(dur)) {
 		us = d * (int)SECS_PER_DAY + S;
+		res.neg = dur.neg || us < 0;
+		us = us >= 0 ? us : -us;
 	}
 
 	if (f.has_week) {
 		/* week shadows days in the hierarchy */
-		res.w = us / (int)SECS_PER_WEEK;
-		us %= (int)SECS_PER_WEEK;
+		res.w = us / SECS_PER_WEEK;
+		us %= SECS_PER_WEEK;
 	}
 	if (f.has_day) {
-		res.d += us / (int)SECS_PER_DAY;
-		us %= (int)SECS_PER_DAY;
+		res.d += us / SECS_PER_DAY;
+		us %= SECS_PER_DAY;
 	}
 	if (f.has_hour) {
-		res.H = us / (long int)SECS_PER_HOUR;
-		us %= (long int)SECS_PER_HOUR;
+		res.H = us / SECS_PER_HOUR;
+		us %= SECS_PER_HOUR;
 	}
 	if (f.has_min) {
 		/* minutes and seconds */
-		res.M = us / (long int)SECS_PER_MIN;
-		us %= (long int)SECS_PER_MIN;
+		res.M = us / SECS_PER_MIN;
+		us %= SECS_PER_MIN;
 	}
 	if (f.has_sec) {
 		res.S = us + __strf_tot_corr(dur);
 	}
-
-	/* just in case the duration iss negative jump through all
-	 * the hoops again, backwards */
-	if (res.w < 0 || res.d < 0 ||
-	    res.H < 0 || res.M < 0 || res.S < 0) {
-		if (0) {
-		fixup_d:
-			res.d = -res.d;
-		fixup_H:
-			res.H = -res.H;
-		fixup_M:
-			res.M = -res.M;
-		fixup_S:
-			res.S = -res.S;
-		} else if (f.has_week) {
-			goto fixup_d;
-		} else if (f.has_day) {
-			goto fixup_H;
-		} else if (f.has_hour) {
-			goto fixup_M;
-		} else if (f.has_min) {
-			goto fixup_S;
+	if (f.has_nano) {
+		if (dur.durtyp == DT_DURNANO) {
+			res.N = dur.dv % NANOS_PER_SEC;
 		}
 	}
 	return res;
@@ -517,7 +512,7 @@ __strfdtdur(
 	/* assign and go */
 	bp = buf;
 	fp = fmt;
-	if (dur.neg) {
+	if (pre.neg) {
 		*bp++ = '-';
 	}
 	for (char *const eo = buf + bsz; *fp && bp < eo;) {
@@ -584,6 +579,10 @@ __strfdtdur(
 			bp += ltostr(bp, eo - bp, pre.m, 2, spec.pad);
 			break;
 
+		case DT_SPFL_N_QTR:
+			bp += ltostr(bp, eo - bp, pre.q, 2, spec.pad);
+			break;
+
 		case DT_SPFL_N_YEAR:
 			bp += ltostr(bp, eo - bp, pre.Y, -1, DT_SPPAD_NONE);
 			break;
@@ -613,6 +612,10 @@ __strfdtdur(
 			bp += ltostr(bp, eo - bp, pre.H, 2, spec.pad);
 			break;
 
+		case DT_SPFL_N_NANO:
+			bp += ltostr(bp, eo - bp, pre.N, 9, DT_SPPAD_ZERO);
+			break;
+
 		default:
 			break;
 		}
@@ -735,6 +738,9 @@ main(int argc, char *argv[])
 		size_t lno = 0;
 		void *pctx;
 
+		/* convert deprecated -S|--skip-illegal */
+		argi->empty_mode_flag += argi->skip_illegal_flag;
+
 		/* no threads reading this stream */
 		__io_setlocking_bycaller(stdout);
 
@@ -757,7 +763,7 @@ main(int argc, char *argv[])
 						dt_io_warn_strpdt(line);
 						rc = 2;
 					}
-					if (argi->skip_illegal_flag) {
+					if (argi->empty_mode_flag) {
 						/* empty line */
 						__io_write("\n", 1U, stdout);
 					}
