@@ -1,6 +1,6 @@
 /*** dzone.c -- convert date/times between timezones
  *
- * Copyright (C) 2011-2018 Sebastian Freundt
+ * Copyright (C) 2011-2022 Sebastian Freundt
  *
  * Author:  Sebastian Freundt <freundt@ga-group.nl>
  *
@@ -50,17 +50,10 @@
 #include "tzraw.h"
 
 struct ztr_s {
-	int32_t trns;
-	int32_t offs;
+	stamp_t trns;
+	int offs;
 };
 
-/* fwd decld in tzraw.h */
-struct ztrdtl_s {
-	int32_t offs;
-	uint8_t dstp;
-	uint8_t abbr;
-} __attribute__((packed));
-
 const char *prog = "dzone";
 static char gbuf[256U];
 
@@ -137,7 +130,7 @@ static size_t
 dz_strftr(char *restrict buf, size_t bsz, struct ztr_s t)
 {
 	static const char fmt[] = "%FT%T%Z";
-	struct dt_dt_s d = dt_dt_initialiser();
+	struct dt_dt_s d = {DT_UNK};
 
 	d.typ = DT_SEXY;
 	d.sexy = t.trns + t.offs;
@@ -157,7 +150,7 @@ dz_write_nxtr(struct zrng_s r, zif_t z, const char *zn)
 	const char *const ep = gbuf + sizeof(gbuf);
 	size_t ntr = zif_ntrans(z);
 
-	if (r.next == INT_MAX) {
+	if (r.next >= STAMP_MAX) {
 		bp += xstrlcpy(bp, never, bp - ep);
 	} else {
 		bp += dz_strftr(bp, ep - bp, (struct ztr_s){r.next, r.offs});
@@ -166,12 +159,12 @@ dz_write_nxtr(struct zrng_s r, zif_t z, const char *zn)
 	bp += xstrlcpy(bp, nindi, bp - ep);
 	if (r.trno + 1U < ntr) {
 		/* thank god there's another one */
-		struct ztrdtl_s zd = zif_trdtl(z, r.trno + 1);
+		stamp_t zdo = zif_troffs(z, r.trno + 1);
 
-		if (r.next == INT_MAX) {
+		if (r.next >= STAMP_MAX) {
 			goto never;
 		}
-		bp += dz_strftr(bp, ep - bp, (struct ztr_s){r.next, zd.offs});
+		bp += dz_strftr(bp, ep - bp, (struct ztr_s){r.next, zdo});
 	} else {
 	never:
 		bp += xstrlcpy(bp, never, bp - ep);
@@ -188,22 +181,22 @@ dz_write_nxtr(struct zrng_s r, zif_t z, const char *zn)
 }
 
 static int
-dz_write_prtr(struct zrng_s r, zif_t UNUSED(z), const char *zn)
+dz_write_prtr(struct zrng_s r, zif_t z, const char *zn)
 {
 	char *restrict bp = gbuf;
 	const char *const ep = gbuf + sizeof(gbuf);
 
 	if (r.trno >= 1) {
 		/* there's one before that */
-		struct ztrdtl_s zd = zif_trdtl(z, r.trno - 1);
+		stamp_t zdo = zif_troffs(z, r.trno - 1);
 
-		bp += dz_strftr(bp, ep - bp, (struct ztr_s){r.prev, zd.offs});
+		bp += dz_strftr(bp, ep - bp, (struct ztr_s){r.prev, zdo});
 	} else {
 		bp += xstrlcpy(bp, never, bp - ep);
 	}
 	/* append prev indicator */
 	bp += xstrlcpy(bp, pindi, bp - ep);
-	if (r.prev == INT_MIN) {
+	if (r.prev <= STAMP_MIN) {
 		bp += xstrlcpy(bp, never, bp - ep);
 	} else {
 		bp += dz_strftr(bp, ep - bp, (struct ztr_s){r.prev, r.offs});
@@ -239,7 +232,7 @@ main(int argc, char *argv[])
 	/* all them datetimes to consider */
 	struct dt_dt_s *d = NULL;
 	size_t nd = 0U;
-	bool trnsp = false;
+	bool trnsp;
 
 	if (yuck_parse(argi, argc, argv)) {
 		rc = 1;
@@ -258,9 +251,8 @@ main(int argc, char *argv[])
 	if (argi->from_zone_arg) {
 		fromz = dt_io_zone(argi->from_zone_arg);
 	}
-	if (argi->next_flag || argi->prev_flag) {
-		trnsp = true;
-	}
+	trnsp = argi->next_flag || argi->prev_flag;
+
 	if (argi->base_arg) {
 		struct dt_dt_s base = dt_strpdt(argi->base_arg, NULL, NULL);
 		dt_set_base(base);
